/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.ViewPart;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;

import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTermImpl;

public class StatisticsSummary extends ViewPart implements
		SarosViewNotification {

	public void createPartControl(Composite parent) {
		statisticsText = new StyledText(parent, SWT.READ_ONLY | SWT.MULTI
				| SWT.WRAP | SWT.V_SCROLL);
		popupMenu = new Menu(parent.getShell(), SWT.POP_UP);
		periodicRefreshItem = new MenuItem(popupMenu, SWT.CHECK);
		periodicRefreshItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.PeriodicRefresh"));
		periodicRefreshItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (periodicRefreshItem.getSelection()) {
					enableReporting();
				} else {
					disableReporting();
				}
			}
		});
		refreshRateItem = new MenuItem(popupMenu, SWT.PUSH);
		refreshRateItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.RefreshRate")
				+ "...");
		refreshRateItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				InputDialog d = new InputDialog(null,
						"Statistics Refresh Rate", "Refresh Rate (seconds):",
						null, null);
				d.setBlockOnOpen(true);
				if (d.open() == Window.OK) {
					activeInteraction.setStatisticsRefreshRate(new Float(d
							.getValue()));
					if (activeInteraction.statisticsReportingEnabled()) {
						enableReporting();
					}
				}
			}
		});
		statisticsText.setMenu(popupMenu);
		IWorkbenchPage page = getSite().getPage();
		if (page == null
				|| !(page.getActiveEditor() instanceof ECLiPSeExecutionEnvironment)) {
			periodicRefreshItem.setEnabled(false);
			refreshRateItem.setEnabled(false);
		} else {
			editorActivated((ECLiPSeInteractionInterface) page
					.getActiveEditor());
		}
	}

	protected void enableReporting() {
		IEditorPart editor = this.getSite().getPage().getActiveEditor();
		if (!(editor instanceof ECLiPSeExecutionEnvironment)) {
			Status status = new Status(
					IStatus.ERROR,
					"Saros",
					0,
					"A Saros ECLiPSe Execution ECLiPSeInteractionInterface must be active.",
					null);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(), "Saros Plugin Error",
					"Unable to enable statistics summary reports.", status);
			return;
		}
		activeInteraction.setStatisticsReportingEnabled(true);
		startCollection(activeInteraction);
	}

	private void disableReporting() {
		IEditorPart editor = this.getSite().getPage().getActiveEditor();
		if (!(editor instanceof ECLiPSeExecutionEnvironment)) {
			Status status = new Status(
					IStatus.ERROR,
					"Saros",
					0,
					"A Saros ECLiPSe Execution ECLiPSeInteractionInterface must be active.",
					null);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(), "Saros Plugin Error",
					"Unable to disable statistics summary reports.", status);
			return;
		}
		activeInteraction.setStatisticsReportingEnabled(false);
		terminateCollection(activeInteraction);
	}

	protected void clearText() {
		statisticsText.replaceTextRange(0, statisticsText.getCharCount(), "");
	}

	public void setFocus() {
		statisticsText.setFocus();
	}

	public void configureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			if (activeInteraction.statisticsReportingEnabled()
					&& !activeInteraction.statisticsCollecting()) {
				startCollection(eclipseInteraction);
			}
		}
	}

	public void editorActivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction
				&& activeInteraction != null) {
			return;
		}
		if (activeInteraction != null) {
			terminateCollection(activeInteraction);
		}

		activeInteraction = eclipseInteraction;
		clearText();
		periodicRefreshItem.setEnabled(true);
		refreshRateItem.setEnabled(true);
		eclipseInteraction.setStatisticsDisplay(statisticsText);
		if (eclipseInteraction.statisticsReportingEnabled()) {
			periodicRefreshItem.setSelection(true);
			startCollection(eclipseInteraction);
		} else {
			periodicRefreshItem.setSelection(false);
			// Hack to make sure the window is refreshed with the statistics
			// from this engine, even though periodic refresh is not enabled
			// - clears the previous engines async output
			startCollection(eclipseInteraction);
			terminateCollection(eclipseInteraction);
		}
	}

	public void editorDeactivated(ECLiPSeInteractionInterface eclipseInteraction) {
		deconfigureEngine(eclipseInteraction);
	}

	public void deconfigureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction) {
			activeInteraction = null;
			clearText();
			periodicRefreshItem.setEnabled(false);
			refreshRateItem.setEnabled(false);
		}
	}

	public void viewClosed(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction) {
			eclipseInteraction.setStatisticsDisplay(null);
			terminateCollection(eclipseInteraction);
		}
	}

	private void startCollection(ECLiPSeInteractionInterface eclipseInteraction) {
		eclipseInteraction.setStatisticsCollecting(true);
		eclipseInteraction.interruptingRpc(new CompoundTermImpl(":", new Atom(
				"tracer_tcl"), new CompoundTermImpl(
				"start_report_stats_text_summary", eclipseInteraction
						.getStatisticsRefreshRate())),
				"Failed to initiate summary statistics collection.");
	}

	private void terminateCollection(
			ECLiPSeInteractionInterface eclipseInteraction) {
		eclipseInteraction.setStatisticsCollecting(false);
		eclipseInteraction.interruptingRpc(new CompoundTermImpl(":", new Atom(
				"tracer_tcl"), new Atom("stop_report_stats_text_summary")),
				"Failed to terminate summary statistics collection.");
	}

	StyledText statisticsText = null;

	private Menu popupMenu = null;

	private ECLiPSeInteractionInterface activeInteraction = null;

	private MenuItem periodicRefreshItem = null;

	private MenuItem refreshRateItem = null;
}
