/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.runtime;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipseclp.saros.core.Saros;
import org.eclipseclp.saros.core.preferences.EnginePreferences;
import org.eclipseclp.saros.core.preferences.LaunchQueryComposite;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceManager;
import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;
import com.parctechnologies.eclipse.EclipseEngine;
import com.parctechnologies.eclipse.EclipseEngineOptions;
import com.parctechnologies.eclipse.EclipseException;
import com.parctechnologies.eclipse.EclipseMultitaskConnection;
import com.parctechnologies.eclipse.EmbeddedEclipse;
import com.parctechnologies.eclipse.MultitaskListener;
import com.parctechnologies.eclipse.OutOfProcessEclipse;

public class Environment {

	public final static int FILE_PROPERTIES = 0;

	public final static int PROJECT_PROPERTIES = 1;

	public final static int PREFRERENCES = 2;

	public final static int PROJECT_LAUNCH_PROPERTIES = 3;

	public Environment(int scope, IProject project, IResource resource) {
		this.scope = scope;
		this.project = project;
		this.resource = resource;
		initialise();
	}

	private void initialise() {
		configureInstallation(scope);
		configureLaunchQuery(scope);
		configureCompilationTargets(scope);
		configureOutputFolders(scope);
	}

	public void reconfigureEngine() {
		// Standalone toplevel - no associated project
		if (project != null) {
			configureInstallation(scope);
		}
		setEngineFlags();
	}

	public EclipseEngine startEngine(boolean useQueues) {
		if (installation == null) {
			return null;
		}

		File enginePath = new File(installation.getInstallationPath());
		if (!enginePath.exists()) {
			return null;
		}

		engineType = PreferenceManager.getEngineType();

		try {
			// ToDo pass engine args
			EclipseEngineOptions eclipseEngineOptions = new EclipseEngineOptions(
					enginePath);
			eclipseEngineOptions.setUseQueues(useQueues);

			switch (engineType) {
			case PreferenceConstants.embedded:
				eclipseEngine = EmbeddedEclipse
						.getInstance(eclipseEngineOptions);
				break;
			case PreferenceConstants.outOfProcess:
				eclipseEngine = new OutOfProcessEclipse(eclipseEngineOptions);
				break;
			default:
				throw new Exception("Unsupported ECLiPSe engine type");
			}
		} catch (Exception e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst initialising ECLiPSe engine: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Runtime Plugin Error", e.getMessage(), status);
			System.err
					.println("Exception caught whilst initialising Saros ECLiPSe "
							+ "engine: " + e.getMessage());
			e.printStackTrace();
			return null;
		}

		setEngineFlags();

		return eclipseEngine;
	}

	public EclipseMultitaskConnection registerMultitask(
			MultitaskListener listener) {
		try {
			multitaskEclipseEngine = eclipseEngine.registerMultitask(listener);
		} catch (Exception e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst initialising ECLiPSe engine: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Runtime Plugin Error", e.getMessage(), status);
			System.err
					.println("Exception caught whilst initialising Saros ECLiPSe "
							+ "engine: " + e.getMessage());
			e.printStackTrace();
			return null;
		}
		return multitaskEclipseEngine;
	}

	public void multitaskConfirm() throws EclipseException, IOException {
		multitaskEclipseEngine.multitaskConfirm();
	}

	public void multitaskTerminate() throws EclipseException, IOException {
		multitaskEclipseEngine.multitaskTerminate();
	}

	private CompoundTerm rpc(CompoundTerm term) throws EclipseException,
			IOException {
		if (multitaskEclipseEngine == null) {
			return eclipseEngine.rpc(term);
		}

		return multitaskEclipseEngine.rpc(term);
	}

	private CompoundTerm rpc(String query) throws EclipseException, IOException {
		if (multitaskEclipseEngine == null) {
			return eclipseEngine.rpc(query);
		}

		return multitaskEclipseEngine.rpc(query);
	}

	private void setEngineFlags() {
		if (project != null) {
			String projectDir = project.getLocation().toOSString()
					+ File.separator;
			String[] libraryPathDirs = installation.getLibraryPath().split(",");
			ArrayList dirs = new ArrayList();
			boolean addedProjectLibDir = false;
			String projectLibDir = projectDir + outputFolders[1];
			for (int dir = 0; dir < libraryPathDirs.length; dir++) {
				dirs.add(libraryPathDirs[dir]);
				if (compilationTargets[0] && !addedProjectLibDir) {
					if (libraryPathDirs[dir].equals(projectLibDir)) {
						addedProjectLibDir = true;
					}
				}
			}
			if (compilationTargets[0] && !addedProjectLibDir) {
				dirs.add(projectLibDir);
			}
			try {
				rpc(new CompoundTermImpl(":", new Atom("tracer_tcl"),
						new CompoundTermImpl("saros_set_library_path", dirs)));
			} catch (Exception e) {
				System.err
						.println("Exception caught whilst initialising Saros ECLiPSe "
								+ "engine library path: " + e.getMessage());
				e.printStackTrace();
			}
		}

		for (int flag = 0; flag < EnginePreferences.flagNames.length; flag++) {
			try {
				switch (EnginePreferences.flagTypes[flag]) {
				case EnginePreferences.ATOM:
					rpc(new CompoundTermImpl(
							"set_flag",
							new Atom(EnginePreferences.flagNames[flag]),
							new Atom(
									installation
											.getStringFlag(EnginePreferences.flagNames[flag]))));
					break;

				case EnginePreferences.STRING:
					rpc(new CompoundTermImpl("set_flag", new Atom(
							EnginePreferences.flagNames[flag]), installation
							.getStringFlag(EnginePreferences.flagNames[flag])));
					break;

				case EnginePreferences.INTEGER:
					rpc(new CompoundTermImpl(
							"set_flag",
							new Atom(EnginePreferences.flagNames[flag]),
							new Integer(
									installation
											.getIntFlag(EnginePreferences.flagNames[flag]))));
					break;
				}
			} catch (Exception e) {
				System.err
						.println("Exception caught whilst configuring Saros ECLiPSe "
								+ "engine: " + e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public void shutdownEngine() {
		if (engineType == PreferenceConstants.outOfProcess) {
			try {
				((OutOfProcessEclipse) eclipseEngine).destroy();
			} catch (IOException e) {
			}
		}
	}

	public void configureInstallation(int scope) {
		switch (scope) {
		case FILE_PROPERTIES:
		case PROJECT_PROPERTIES:
		case PROJECT_LAUNCH_PROPERTIES:
			installation = configureProjectInstallation();
			if (installation == null) {
				installation = configureDefaultInstallation();
			}
			break;
		case PREFRERENCES:
		default:
			installation = configureDefaultInstallation();
			break;
		}
	}

	private EnginePreferences configureProjectInstallation() {
		EnginePreferences installation = null;
		if (project == null) {
			return null;
		}

		try {
			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.IsProjectSpecificInstallation))
					.equals(PreferenceConstants.True)) {
				String installationName = project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.ECLiPSeInstallationName));
				installation = PreferenceManager
						.getInstallationByName(installationName);
			}

			if (installation == null) {
				installation = configureDefaultInstallation();
			}

			// Get any project specific properties

			if (project
					.getPersistentProperty(
							new QualifiedName(
									PreferenceConstants.SarosPlugin,
									PreferenceConstants.IsProjectSpecificEngineSettings))
					.equals(PreferenceConstants.True)) {
				installation.setFlag(PreferenceConstants.afterEventTimer,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.afterEventTimer)));
				installation.setFlag(PreferenceConstants.allDynamic, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.allDynamic)));
				installation.setFlag(PreferenceConstants.brealExceptions,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.brealExceptions)));
				installation.setFlag(PreferenceConstants.coroutine, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.coroutine)));
				installation.setFlag(PreferenceConstants.debugging, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.debugging)));
				installation.setFlag(PreferenceConstants.debugCompile, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.debugCompile)));
				installation.setFlag(PreferenceConstants.enableInterrupts,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.enableInterrupts)));
				installation.setFlag(PreferenceConstants.goalExpansion, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.goalExpansion)));
				installation.setFlag(PreferenceConstants.macroExpansion,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.macroExpansion)));
				installation.setFlag(PreferenceConstants.preferRationals,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.preferRationals)));
				installation.setFlag(PreferenceConstants.variableNames, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.variableNames)));
			}

			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.IsProjectSpecificGCsettings))
					.equals(PreferenceConstants.True)) {
				installation.setFlag(PreferenceConstants.gc, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.gc)));
				installation.setFlag(PreferenceConstants.gcPolicy, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.gcPolicy)));
				installation.setFlag(PreferenceConstants.gcInterval,
						new Integer(project
								.getPersistentProperty(new QualifiedName(
										PreferenceConstants.SarosPlugin,
										PreferenceConstants.gcInterval))));
				installation.setFlag(PreferenceConstants.gcIntervalDict,
						new Integer(project
								.getPersistentProperty(new QualifiedName(
										PreferenceConstants.SarosPlugin,
										PreferenceConstants.gcIntervalDict))));
			}

			if (project
					.getPersistentProperty(
							new QualifiedName(
									PreferenceConstants.SarosPlugin,
									PreferenceConstants.IsProjectSpecificOutputOptions))
					.equals(PreferenceConstants.True)) {
				installation.setFlag(PreferenceConstants.outputMode, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.outputMode)));
				installation.setFlag(PreferenceConstants.printDepth,
						new Integer(project
								.getPersistentProperty(new QualifiedName(
										PreferenceConstants.SarosPlugin,
										PreferenceConstants.printDepth))));
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving project properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
			installation = null;
		}
		return installation;
	}

	private EnginePreferences configureDefaultInstallation() {
		installation = PreferenceManager.getDefaultInstallation();
		return installation;
	}

	public void configureLaunchQuery(int scope) {
		switch (scope) {
		case FILE_PROPERTIES:
			configureFileLaunchQuery();
			if (!hasLaunchQuery) {
				configureProjectLaunchQuery();
			}
			break;
		case PROJECT_PROPERTIES:
			configureProjectLaunchQuery();
		case PROJECT_LAUNCH_PROPERTIES:
			return;
		case PREFRERENCES:
		default:
			hasLaunchQuery = false;
			break;
		}
	}

	private void configureProjectLaunchQuery() {
		if (project == null) {
			return;
		}

		try {
			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.HasLaunchQuery)).equals(
					PreferenceConstants.True)) {
				hasLaunchQuery = true;
				launchfileName = project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchFile));
				launchModuleName = project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchModule));
				launchQuery = project.getPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.LaunchQuery));
				launchRunMethod = project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.LaunchRunMode));
			} else {
				hasLaunchQuery = false;
				launchfileName = "";
				launchModuleName = "";
				launchQuery = "";
				launchRunMethod = "";
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving resource properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
			hasLaunchQuery = false;
			launchfileName = "";
			launchModuleName = "";
			launchQuery = "";
			launchRunMethod = "";
		}
	}

	private void configureFileLaunchQuery() {
		try {
			if (resource.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.FileLaunchQuery)).equals(
					PreferenceConstants.True)) {
				if (resource.getPersistentProperty(
						new QualifiedName(PreferenceConstants.SarosPlugin,
								PreferenceConstants.HasLaunchQuery)).equals(
						PreferenceConstants.True)) {
					hasLaunchQuery = true;
					launchfileName = resource
							.getPersistentProperty(new QualifiedName(
									PreferenceConstants.SarosPlugin,
									PreferenceConstants.LaunchFile));
					launchModuleName = resource
							.getPersistentProperty(new QualifiedName(
									PreferenceConstants.SarosPlugin,
									PreferenceConstants.LaunchModule));
					launchQuery = resource
							.getPersistentProperty(new QualifiedName(
									PreferenceConstants.SarosPlugin,
									PreferenceConstants.LaunchQuery));
					launchRunMethod = resource
							.getPersistentProperty(new QualifiedName(
									PreferenceConstants.SarosPlugin,
									PreferenceConstants.LaunchRunMode));
					return;
				}
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving resource properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
		hasLaunchQuery = false;
		launchfileName = "";
		launchModuleName = "";
		launchQuery = "";
		launchRunMethod = "";
	}

	public void setLaunchQuery(boolean hasLaunchQuery, String launchFile,
			String launchModule, String runMethod, String launchQuery) {
		this.hasLaunchQuery = hasLaunchQuery;
		this.launchfileName = launchFile;
		this.launchModuleName = launchModule;
		this.launchQuery = launchQuery;
		this.launchRunMethod = runMethod;
	}

	public void configureCompilationTargets(int scope) {
		switch (scope) {
		case FILE_PROPERTIES:
		case PROJECT_PROPERTIES:
		case PROJECT_LAUNCH_PROPERTIES:
			compilationTargets = configureProjectCompilationTargets();
			if (compilationTargets == null) {
				compilationTargets = configureDefaultCompilationTargets();
			}
			break;
		case PREFRERENCES:
		default:
			compilationTargets = configureDefaultCompilationTargets();
			break;
		}
	}

	private boolean[] configureDefaultCompilationTargets() {
		return PreferenceManager.getOutputCompilationTargets();
	}

	private boolean[] configureProjectCompilationTargets() {
		if (project == null) {
			return null;
		}

		try {
			if (project
					.getPersistentProperty(
							new QualifiedName(
									PreferenceConstants.SarosPlugin,
									PreferenceConstants.IsProjectSpecificCompilationTarget))
					.equals(PreferenceConstants.True)) {
				boolean[] compilationTargets = new boolean[3];
				if (project.getPersistentProperty(
						new QualifiedName(PreferenceConstants.SarosPlugin,
								PreferenceConstants.ECLiPSeFcompileTarget))
						.equals(PreferenceConstants.True)) {
					compilationTargets[0] = true;
				} else {
					compilationTargets[0] = false;
				}
				if (project.getPersistentProperty(
						new QualifiedName(PreferenceConstants.SarosPlugin,
								PreferenceConstants.ECLiPSeIcompileTarget))
						.equals(PreferenceConstants.True)) {
					compilationTargets[1] = true;
				} else {
					compilationTargets[1] = false;
				}

				if (project.getPersistentProperty(
						new QualifiedName(PreferenceConstants.SarosPlugin,
								PreferenceConstants.ECLiPSeEciToHtml)).equals(
						PreferenceConstants.True)) {
					compilationTargets[2] = true;
				} else {
					compilationTargets[2] = false;
				}
				return compilationTargets;
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving resource properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
		return null;
	}

	public void configureOutputFolders(int scope) {
		switch (scope) {
		case FILE_PROPERTIES:
		case PROJECT_PROPERTIES:
		case PROJECT_LAUNCH_PROPERTIES:
			outputFolders = configureProjectOutputFolders();
			if (outputFolders == null) {
				outputFolders = configureDefaultOutputFolders();
			}

			break;
		case PREFRERENCES:
		default:
			outputFolders = configureDefaultOutputFolders();
			break;
		}
	}

	private String[] configureDefaultOutputFolders() {
		return PreferenceManager.getOutputFolders();
	}

	private String[] configureProjectOutputFolders() {
		if (project == null) {
			return null;
		}

		String[] outputFolders = new String[4];
		try {
			outputFolders[0] = project.getPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.ECLiPSeSourceFolderName));
			outputFolders[1] = project.getPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.ECLiPSeObjectFolderName));
			outputFolders[2] = project.getPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.ECLiPSeDocFolderName));
			outputFolders[3] = project.getPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.ECLiPSeHtmlFolderName));
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving resource properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
			for (int cnt = 0; cnt < outputFolders.length; cnt++) {
				outputFolders[cnt] = "";
			}
		}
		return outputFolders;
	}

	public void compile() {
		try {
			if (hasLaunchQuery) {
				if (!launchfileName.equals("")) {
					if (!outputFolders[0].equals("")) {
						launchfileName = outputFolders[0] + "/"
								+ launchfileName;
					}
					IFile file = project.getFile(launchfileName);
					rpc(new CompoundTermImpl(":", new Atom("tracer_tcl"),
							new CompoundTermImpl("saros_compile", file
									.getRawLocation().toOSString())));
				} else if (!launchModuleName.equals("")) {
					rpc(new CompoundTermImpl("lib", new Atom(launchModuleName)));
				}
			} else if (resource != null && resource instanceof IFile) {
				rpc(new CompoundTermImpl(":", new Atom("tracer_tcl"),
						new CompoundTermImpl("saros_compile", resource
								.getRawLocation().toOSString())));
			}
		} catch (Exception e) {
			System.err
					.println("Exception caught whilst compiling Saros ECLiPSe "
							+ "launch query: " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void launchQuery() throws EclipseException, IOException {
		if (hasLaunchQuery && !launchQuery.equals("")) {
			String query = LaunchQueryComposite
					.getRunMethodData(launchRunMethod)
					+ "(";
			if (!launchModuleName.equals("")) {
				query += launchModuleName + ":";
			}
			query += launchQuery + ")";
			rpc(query);
		}
	}

	public void setCompilationTargets(boolean[] compilationTargets) {
		this.compilationTargets = compilationTargets;
	}

	public boolean[] getCompilationTargets() {
		return compilationTargets;
	}

	public boolean hasLaunchQuery() {
		return hasLaunchQuery;
	}

	public void setHasLaunchQuery(boolean hasLaunchQuery) {
		this.hasLaunchQuery = hasLaunchQuery;
	}

	public EnginePreferences getInstallation() {
		return installation;
	}

	public void setInstallation(EnginePreferences installation) {
		this.installation = installation;
	}

	public String getLaunchfileName() {
		return launchfileName;
	}

	public void setLaunchfileName(String launchfileName) {
		this.launchfileName = launchfileName;
	}

	public String getLaunchModuleName() {
		return launchModuleName;
	}

	public void setLaunchModuleName(String launchModuleName) {
		this.launchModuleName = launchModuleName;
	}

	public String getLaunchQuery() {
		return launchQuery;
	}

	public void setLaunchQuery(String launchQuery) {
		this.launchQuery = launchQuery;
	}

	public String getLaunchRunMethod() {
		return launchRunMethod;
	}

	public void setLaunchRunMethod(String launchRunMethod) {
		this.launchRunMethod = launchRunMethod;
	}

	public String[] getOutputFolders() {
		return outputFolders;
	}

	public void setOutputFolders(String[] outputFolders) {
		this.outputFolders = outputFolders;
	}

	public int getScope() {
		return scope;
	}

	public void setScope(int scope) {
		this.scope = scope;
	}

	public int getEngineType() {
		return engineType;
	}

	public IProject getProject() {
		return project;
	}

	private int scope;

	private EnginePreferences installation = null;

	private boolean hasLaunchQuery = false;

	private String launchfileName = "";

	private String launchModuleName = "";

	private String launchQuery = "";

	private String launchRunMethod = "";

	private boolean[] compilationTargets = null;

	private String[] outputFolders = null;

	private IProject project = null;

	private IResource resource = null;

	private EclipseEngine eclipseEngine = null;

	private EclipseMultitaskConnection multitaskEclipseEngine = null;

	private int engineType = 1;
}
